// =============================================================================
// Copyright (C) MACNICA inc.
// =============================================================================
// FILE NAME      : i2c_init_ctl.v 
// DEPARTMENT     : MACNICA Strategic Technology Group
// AUTHOR         : Youhei Nagano
// AUTHOR'S EMAIL : Nagano-y@macnica.co.jp
// =============================================================================
// KEYWORDS :  PRJ-000108
// =============================================================================
// PURPOSE  :  Scan Conversion IP Reference Design for Alvix
// =============================================================================
// COMMENTS : Registered by Nagano ,
// =============================================================================

module  i2c_init_ctl (
        clk,
        rst_n,
        srst,
        
        wt_d_req_i,
        rd_data_i,
        rd_d_valid_i,
        
        busy_i,
        error_i,
        
        control_reg_o,
        dev_sel_o,
        w_r_adr_o,
        b_size_o,
        start_o,
        wt_data_o,
        
        init_end_o,
        err_status_o
        );
    
/****************************************************************/
/*****                  parameter definition                *****/
/****************************************************************/
    
    parameter I2CM_IDLE   = 4'b0000;
    parameter I2CM_INIT   = 4'b0001;
    parameter I2CM_IDLE2  = 4'b0010;
    parameter I2CM_WAIT1  = 4'b1110;
    parameter I2CM_WAIT2  = 4'b1111;
    
    
    
/****************************************************************/
/*****                  port declaration                    *****/
/****************************************************************/
    
    input                               clk;            //
    input                               rst_n;          //
    input                               srst;           //
    
    input                               wt_d_req_i;     //
    input   [7:0]                       rd_data_i;      //
    input                               rd_d_valid_i;   //
    
    input                               busy_i;         //
    input                               error_i;        //
    
    output  [6:0]                       control_reg_o;  //
    output  [9:0]                       dev_sel_o;      //
    output  [23:0]                      w_r_adr_o;      //
    output  [23:0]                      b_size_o;       //
    output                              start_o;        //
    output  [7:0]                       wt_data_o;      //
    
    output                              init_end_o;     //
    output                              err_status_o;   //
    
    
    
/****************************************************************/
/*****                  wire signal assignment              *****/
/****************************************************************/
    
    reg     [3:0]                       i2c_state;      //
    reg     [3:0]                       i2c_main_state; //
    
    reg     [6:0]                       control_reg_ff; //
    reg     [9:0]                       dev_sel_ff;     //
    reg     [23:0]                      w_r_adr_ff;     //
    reg     [23:0]                      b_size_ff;      //
    reg                                 start_ff;       //
    reg     [7:0]                       wt_data_ff;     //
    
    reg                                 init_end_ff;
    reg                                 err_flg_ff;     //
    
    reg     [7:0]                       access_cnt_ff;  //
    
    reg     [44:0]                      init_data;      //
    
    
    
/****************************************************************/
/*****                  function description                *****/
/****************************************************************/
    
    assign control_reg_o    = control_reg_ff;
    assign dev_sel_o        = dev_sel_ff;
    assign w_r_adr_o        = w_r_adr_ff;
    assign b_size_o         = b_size_ff;
    assign start_o          = start_ff;
    assign wt_data_o        = wt_data_ff;
    
    assign init_end_o       = init_end_ff;
    assign err_status_o     = err_flg_ff;
    
    /************************************/
    /* i2c master control state machine */
    /************************************/
    
    always @( posedge clk or negedge rst_n ) begin
        if ( ~rst_n ) begin
            i2c_state       <= I2CM_IDLE;
            i2c_main_state  <= I2CM_IDLE;
            control_reg_ff  <= 7'b0_01_0_100;
            dev_sel_ff      <= 10'b000_0000000;
            w_r_adr_ff      <= 24'h00_0000;
            b_size_ff       <= 24'h00_0001;
            start_ff        <= 1'b0;
            wt_data_ff      <= 8'h00;
            access_cnt_ff   <= 8'h00;
            init_end_ff     <= 1'b0 ;
        end
        else if ( srst ) begin
            i2c_state       <= I2CM_IDLE;
            i2c_main_state  <= I2CM_IDLE;
            control_reg_ff  <= 7'b0_01_0_100;
            dev_sel_ff      <= 10'b000_0000000;
            w_r_adr_ff      <= 24'h00_0000;
            b_size_ff       <= 24'h00_0001;
            start_ff        <= 1'b0;
            wt_data_ff      <= 8'h00;
            access_cnt_ff   <= 8'h00;
            init_end_ff     <= 1'b0 ;
        end
        else begin
            case ( i2c_state )
                
                I2CM_IDLE      : begin
                    i2c_state       <= I2CM_INIT;
                    i2c_main_state  <= I2CM_INIT;
                end
                
                I2CM_INIT      : begin
                    if ( init_data[41:32] == 10'h000 ) begin
                        i2c_state       <= I2CM_IDLE2;
                        i2c_main_state  <= I2CM_IDLE2;
                        access_cnt_ff   <= 8'h00;
                    end
                    else begin
                        i2c_state       <= I2CM_WAIT1;
                        control_reg_ff  <= { 1'b0 , init_data[43:42] , init_data[44] , 3'b100 };
                        dev_sel_ff      <= init_data[41:32];
                        w_r_adr_ff      <= init_data[31:8];
                        b_size_ff       <= 24'h00_0001;
                        start_ff        <= 1'b1;
                        wt_data_ff      <= init_data[7:0];
                    end
                end
                
                I2CM_IDLE2     : begin
                    init_end_ff <= 1'b1 ;
                end
                
                
                
                I2CM_WAIT1     : begin
                    start_ff        <= 1'b0;
                    if ( busy_i ) begin
                        i2c_state       <= I2CM_WAIT2;
                        access_cnt_ff   <= access_cnt_ff + 1'b1;
                    end
                end
                
                I2CM_WAIT2     : begin
                    if ( ~busy_i ) begin
                        i2c_state       <= i2c_main_state;
                    end
                end
                
                default         : begin
                    i2c_state       <= I2CM_IDLE;
                    i2c_main_state  <= I2CM_IDLE;
                    control_reg_ff  <= 7'b0_01_0_100;
                    dev_sel_ff      <= 10'b000_0000000;
                    w_r_adr_ff      <= 24'h00_0000;
                    b_size_ff       <= 24'h00_0001;
                    start_ff        <= 1'b0;
                    wt_data_ff      <= 8'h00;
                    access_cnt_ff   <= 8'h00;
                    init_end_ff     <= 1'b0 ;
                end
                
            endcase
        end
    end
    
    /*********************************/
    /* initial access data selection */
    /*********************************/
    
    always @( * ) begin
        case ( access_cnt_ff )
                                    // devsize adrsize  dev adr   mem adr      wr data
            8'h00   : init_data     <= { 1'b0 , 2'b10 , 10'h038 , 24'h004000 , 8'b0000_0001 } ;// 
            8'h01   : init_data     <= { 1'b0 , 2'b10 , 10'h038 , 24'h004002 , 8'b0000_0000 } ;// 
            8'h02   : init_data     <= { 1'b0 , 2'b10 , 10'h038 , 24'h004003 , 8'b1111_1101 } ;// 
            8'h03   : init_data     <= { 1'b0 , 2'b10 , 10'h038 , 24'h004004 , 8'b0000_0000 } ;// 
            8'h04   : init_data     <= { 1'b0 , 2'b10 , 10'h038 , 24'h004005 , 8'b0000_1100 } ;// 
            8'h05   : init_data     <= { 1'b0 , 2'b10 , 10'h038 , 24'h004006 , 8'b0001_0000 } ;// 
            8'h06   : init_data     <= { 1'b0 , 2'b10 , 10'h038 , 24'h004007 , 8'b0000_0000 } ;// 
            8'h07   : init_data     <= { 1'b0 , 2'b10 , 10'h038 , 24'h004008 , 8'b0000_0000 } ;// 
            8'h08   : init_data     <= { 1'b0 , 2'b10 , 10'h038 , 24'h00400A , 8'b0111_0001 } ;// Mixer1 Enable LINP:0dB
            8'h09   : init_data     <= { 1'b0 , 2'b10 , 10'h038 , 24'h00400B , 8'b0000_0000 } ;// 
            8'h0a   : init_data     <= { 1'b0 , 2'b10 , 10'h038 , 24'h00400C , 8'b0111_0001 } ;// Mixer2 Enable RINP:0dB
            8'h0b   : init_data     <= { 1'b0 , 2'b10 , 10'h038 , 24'h00400D , 8'b0000_0000 } ;// 
            8'h0c   : init_data     <= { 1'b0 , 2'b10 , 10'h038 , 24'h00400E , 8'b0100_0000 } ;// 
            8'h0d   : init_data     <= { 1'b0 , 2'b10 , 10'h038 , 24'h00400F , 8'b0100_0000 } ;// 
            8'h0e   : init_data     <= { 1'b0 , 2'b10 , 10'h038 , 24'h004010 , 8'b0000_0000 } ;// 
            8'h0f   : init_data     <= { 1'b0 , 2'b10 , 10'h038 , 24'h004011 , 8'b0001_0011 } ;// 
            8'h10   : init_data     <= { 1'b0 , 2'b10 , 10'h038 , 24'h004012 , 8'b0011_1100 } ;// 
            8'h11   : init_data     <= { 1'b0 , 2'b10 , 10'h038 , 24'h004013 , 8'b0000_0110 } ;// 
            8'h12   : init_data     <= { 1'b0 , 2'b10 , 10'h038 , 24'h004014 , 8'b0110_0000 } ;// 
            8'h13   : init_data     <= { 1'b0 , 2'b10 , 10'h038 , 24'h004015 , 8'b1000_0000 } ;// 
            8'h14   : init_data     <= { 1'b0 , 2'b10 , 10'h038 , 24'h004016 , 8'b0010_0000 } ;// 
            8'h15   : init_data     <= { 1'b0 , 2'b10 , 10'h038 , 24'h004017 , 8'b0000_0000 } ;// 
            8'h16   : init_data     <= { 1'b0 , 2'b10 , 10'h038 , 24'h004018 , 8'b0000_0000 } ;// 
            8'h17   : init_data     <= { 1'b0 , 2'b10 , 10'h038 , 24'h004019 , 8'b0000_0011 } ;// 
            8'h18   : init_data     <= { 1'b0 , 2'b10 , 10'h038 , 24'h00401A , 8'b0000_0000 } ;// 
            8'h19   : init_data     <= { 1'b0 , 2'b10 , 10'h038 , 24'h00401B , 8'b0000_0000 } ;// 
            8'h1a   : init_data     <= { 1'b0 , 2'b10 , 10'h038 , 24'h00401C , 8'b0010_0001 } ;// 
            8'h1b   : init_data     <= { 1'b0 , 2'b10 , 10'h038 , 24'h00401D , 8'b0000_0000 } ;// 
            8'h1c   : init_data     <= { 1'b0 , 2'b10 , 10'h038 , 24'h00401E , 8'b0100_0001 } ;// 
            8'h1d   : init_data     <= { 1'b0 , 2'b10 , 10'h038 , 24'h00401F , 8'b0000_0000 } ;// 
            8'h1e   : init_data     <= { 1'b0 , 2'b10 , 10'h038 , 24'h004020 , 8'b0000_0101 } ;// 
            8'h1f   : init_data     <= { 1'b0 , 2'b10 , 10'h038 , 24'h004021 , 8'b0001_0001 } ;// 
            8'h20   : init_data     <= { 1'b0 , 2'b10 , 10'h038 , 24'h004022 , 8'b0000_0000 } ;// 
            8'h21   : init_data     <= { 1'b0 , 2'b10 , 10'h038 , 24'h004023 , 8'b0000_0000 } ;// 
            8'h22   : init_data     <= { 1'b0 , 2'b10 , 10'h038 , 24'h004024 , 8'b0000_0000 } ;// 
            8'h23   : init_data     <= { 1'b0 , 2'b10 , 10'h038 , 24'h004025 , 8'b1110_0110 } ;// 
            8'h24   : init_data     <= { 1'b0 , 2'b10 , 10'h038 , 24'h004026 , 8'b1110_0110 } ;// 
            8'h25   : init_data     <= { 1'b0 , 2'b10 , 10'h038 , 24'h004027 , 8'b0000_0000 } ;// 
            8'h26   : init_data     <= { 1'b0 , 2'b10 , 10'h038 , 24'h004028 , 8'b0000_0000 } ;// 
            8'h27   : init_data     <= { 1'b0 , 2'b10 , 10'h038 , 24'h004029 , 8'b0000_0011 } ;// 
            8'h28   : init_data     <= { 1'b0 , 2'b10 , 10'h038 , 24'h00402A , 8'b0000_0011 } ;// 
            8'h29   : init_data     <= { 1'b0 , 2'b10 , 10'h038 , 24'h00402B , 8'b0000_0000 } ;// 
            8'h2a   : init_data     <= { 1'b0 , 2'b10 , 10'h038 , 24'h00402C , 8'b0000_0000 } ;// 
            8'h2b   : init_data     <= { 1'b0 , 2'b10 , 10'h038 , 24'h00402D , 8'b1010_1010 } ;// 
            8'h2c   : init_data     <= { 1'b0 , 2'b10 , 10'h038 , 24'h00402F , 8'b1010_0000 } ;// 
            8'h2d   : init_data     <= { 1'b0 , 2'b10 , 10'h038 , 24'h004030 , 8'b0000_0000 } ;// 
            8'h2e   : init_data     <= { 1'b0 , 2'b10 , 10'h038 , 24'h004031 , 8'b0000_0000 } ;// 
            8'h2f   : init_data     <= { 1'b0 , 2'b10 , 10'h038 , 24'h004036 , 8'b0000_0011 } ;// 
            default : init_data     <= 45'b00_0_0000000000_000000000000000000000000_00000000;
                                    // devsize adrsize  dev adr   mem adr      wr data
        endcase
    end
    
    /**********************/
    /* i2c error detector */
    /**********************/
    
    always @( posedge clk or negedge rst_n ) begin
        if ( ~rst_n ) begin
            err_flg_ff      <= 1'b0;
        end
        else if ( error_i ) begin
            err_flg_ff      <= 1'b1;
        end
    end
    
    
    
endmodule
