// =============================================================================
// Copyright (C) MACNICA inc.
// =============================================================================
// FILE NAME	: avb_frm_wt_async_fifo.v
// DEPARTMENT	: MACNICA Strategic Technology Group
// AUTHOR		: Yukinobu Sugino
// DATE			: 08/22/2013
// =============================================================================
// DESCRIPTION	: async fifo
//
// =============================================================================
// REVISION HISTORY	:
//	v1.0	: Initial version release
//
// =============================================================================
// PARAMETERS	:
//		FIFO_DEPTH		: FIFO depth
//		DATA_BIT_WIDTH	: data bit width
//		WATERMARK_W		: watermark of asserting almost full flag
//						: ( word count of remaining writable space )
//		WATERMARK_R		: watermark of asserting almost empty flag
//						: ( word count of remaining readable space )
//		RETIME			: read data retiming setting 0:no retiming / 1:retiming
//
// =============================================================================
// I/O PORTS	:
//		wclk		: write clock
//		rclk		: read clock
//		rst_n		: async reset
//		srst		: sync reset
//		wen_i		: write enable
//		ren_i		: read enable
//		wdata_i		: write data
//		rdata_o		: read data
//		a_empty_o	: almost empty flag
//		empty_o		: empty flag
//		a_full_o	: almost full flag
//		full_o		: full flag
//
// =============================================================================
`timescale 1ps/1ps
`default_nettype none


module avb_frm_wt_async_fifo (
	wclk		,	//	write clock
	rclk		,	//	read clock
	rst_n		,	//	async reset
	srst		,	//	sync reset
	wen_i		,	//	write enable
	ren_i		,	//	read enable
	wdata_i		,	//	write data
	rdata_o		,	//	read data

	a_empty_o	,	//	almost empty flag
	empty_o		,	//	empty flag
	a_full_o	,	//	almost full flag
	full_o			//	full flag
) ;

// =============================================================================
// PARAMETER DEFINITION
// =============================================================================

	// ---------------------------------------------------------------------
	// Below parameters have to be defined from upper module
	// ---------------------------------------------------------------------
	parameter	FIFO_DEPTH				=  8 ;
	parameter	DATA_BIT_WIDTH			= 16 ;
	parameter	WATERMARK_W				=  1 ;
	parameter	WATERMARK_R				=  1 ;
	parameter	RETIME					=  0 ;

	// ---------------------------------------------------------------------
	// Please do not change the following parameters
	// ---------------------------------------------------------------------
	parameter	DATA_DEPTH				= 2**FIFO_DEPTH ;

// =============================================================================
// PORT DECLARATION
// =============================================================================
	input	wire								wclk		; //  write clock
	input	wire								rclk		; //  read clock
	input	wire								rst_n		; //  async reset
	input	wire								srst		; //  sync reset

	input	wire								wen_i		; //  write enable
	input	wire								ren_i		; //  read enable
	input	wire	[ DATA_BIT_WIDTH-1 : 0 ]	wdata_i		; //  write data
	output	wire	[ DATA_BIT_WIDTH-1 : 0 ]	rdata_o		; //  read data
	output	wire								a_empty_o	; //  almost empty flag
	output	wire								empty_o		; //  empty flag
	output	wire								a_full_o	; //  almost full flag
	output	wire								full_o		; //  full flag

// =============================================================================
// reg/wire declaration
// =============================================================================

	wire								ram_wen   ;
	wire								ram_ren   ;
	wire	[	  FIFO_DEPTH-1 : 0 ]	ram_waddr ;
	wire	[	  FIFO_DEPTH-1 : 0 ]	ram_raddr ;
	wire	[ DATA_BIT_WIDTH-1 : 0 ]	ram_wdata ;
	wire	[ DATA_BIT_WIDTH-1 : 0 ]	ram_rdata ;

// =============================================================================
// FUNCTION DESCRIPTION
// =============================================================================

// instance

	avb_frm_wt_async_fifo_core
		#(
			.FIFO_DEPTH 		( FIFO_DEPTH		) ,
			.DATA_BIT_WIDTH 	( DATA_BIT_WIDTH	) ,
			.WATERMARK_W		( WATERMARK_W		) ,
			.WATERMARK_R		( WATERMARK_R		) ,
			.RETIME 			( RETIME			) 
			)
		u_fifo_core (
			.wt_clk 			( wclk 			) ,
			.rd_clk 			( rclk 			) ,
			.rst_n				( rst_n			) ,
			.srst				( srst 			) ,

			.wt_en_in			( wen_i			) ,
			.rd_en_in			( ren_i			) ,
			.data_in			( wdata_i		) ,
			.data_out			( rdata_o		) ,
			.a_empty_out		( a_empty_o		) ,
			.empty_out			( empty_o		) ,
			.a_full_out 		( a_full_o 		) ,
			.full_out			( full_o		) ,

			.ram_wt_en_out		( ram_wen		) ,
			.ram_rd_en_out		( ram_ren		) ,
			.ram_wt_adr_out		( ram_waddr		) ,
			.ram_rd_adr_out		( ram_raddr		) ,
			.ram_wt_data_out	( ram_wdata		) ,
			.ram_rd_data_in 	( ram_rdata		) 
		) ;

	avb_frm_wt_ram
		#(
			.ADR_BIT	( FIFO_DEPTH		) ,
			.DATA_BIT	( DATA_BIT_WIDTH	) ,
			.DATA_DEPTH	( DATA_DEPTH		) 
			)
		u_ram (
			.wclk		( wclk			) ,
			.rclk		( rclk			) ,
			.wadr_i		( ram_waddr 	) ,
			.radr_i		( ram_raddr 	) ,
			.wen_i		( ram_wen		) ,
			.ren_i		( ram_ren		) ,
			.wdata_i	( ram_wdata 	) ,
			.rdata_o	( ram_rdata 	) 
		) ;

endmodule

`default_nettype wire
