// =============================================================================
// Copyright (C) MACNICA inc.
// =============================================================================
// FILE NAME	: avb_pkt_asm_div_radix2.v
// DEPARTMENT	: MACNICA Strategic Technology Group
// AUTHOR		: Yuichi Shinoda
// DATE			: 08/22/2013
// =============================================================================
// DESCRIPTION	: 2's base divider component
//
// =============================================================================
// REVISION HISTORY	:
//	v1.0	: Initial version release
//
// =============================================================================
// PARAMETERS	:
//		DVEND 	: dividend bit width 2 ~ 32
//		DVSOR 	: divisor bit width 2 ~ 32
//
// =============================================================================
// I/O PORTS	:
//		clk			: clock for all circuit
//		rst_n		: asynchronous reset ( low active )
//		srst		: synchronous reset
//		enable	 	: clock enable
//		
//		valid_i		: data valid of input data
//		dividend_i	: dividend data
//		divisor_i	: divisor data
//		
//		valid_o		: data valid of output data
//		dividend_o	: shifted dividend data
//		divisor_o	: divisor data
//
// =============================================================================
`timescale 1ps/1ps
`default_nettype none

module	avb_pkt_asm_div_radix2 (
	clk			,
	rst_n		,
	srst		,
	enable		,
	
	valid_i		,
	dividend_i	,
	divisor_i	,
	
	valid_o		,
	dividend_o	,
	divisor_o
) ;
	
// =============================================================================
// PARAMETER DEFINITION
// =============================================================================

	// ---------------------------------------------------------------------
	// Below parameters have to be defined from upper module
	// ---------------------------------------------------------------------
	parameter DVEND 					= 8 ;	// dividend bit width 2 ~ 32
	parameter DVSOR 					= 8 ;	// divisor bit width 2 ~ 32

	// ---------------------------------------------------------------------
	// Please do not change the following parameters
	// ---------------------------------------------------------------------
	parameter INT_DVEND 				= DVEND + DVSOR ;
	
// =============================================================================
// PORT DECLARATION
// =============================================================================

	input	wire							clk			;	// clock for all circuit
	input	wire							rst_n		;	// asynchronous reset ( low active )
	input	wire							srst		;	// synchronous reset
	input	wire							enable		;	// clock enable
	
	input	wire							valid_i		;	// data valid of input data
	input	wire	[ INT_DVEND-1 : 0 ]		dividend_i	;	// dividend data
	input	wire	[     DVSOR-1 : 0 ]		divisor_i	;	// divisor data
	
	output	wire							valid_o		;	// data valid of output data
	output	wire	[ INT_DVEND-1 : 0 ]		dividend_o	;	// shifted dividend data
	output	wire	[     DVSOR-1 : 0 ]		divisor_o	;	// divisor data
	
// =============================================================================
// REG / WIRE DECLARATION
// =============================================================================
	
	wire	[     DVSOR+1 : 0 ]		diff_mult			;
	wire							diff_flag			;
	
	reg 							valid_ff			;
	reg 	[ INT_DVEND-1 : 0 ]		dividend_lat_ff		;
	reg 	[     DVSOR-1 : 0 ]		divisor_lat_ff		;
	
	reg 	[ INT_DVEND-1 : 0 ]		dividend_out_comb	;
	
// =============================================================================
// FUNCTION DESCRIPTION
// =============================================================================
	
	assign diff_mult			= { 1'b0 , dividend_lat_ff[INT_DVEND-1:DVEND-1] } - { 2'b00 , divisor_lat_ff } ;
	assign diff_flag			= diff_mult[DVSOR+1] ;
	
	always @( * ) begin
		case ( diff_flag ) //synopsys full_case
			1'b0 :		begin
				dividend_out_comb	<= { diff_mult[DVSOR-1:0] , dividend_lat_ff[DVEND-2:0] , 1'b1 } ;
			end
			1'b1 :		begin
				dividend_out_comb	<= { dividend_lat_ff[INT_DVEND-2:0] , 1'b0 } ;
			end
		endcase
	end
	
	always @( posedge clk or negedge rst_n ) begin
		if ( !rst_n ) begin
			valid_ff		<= 1'b0 ;
			dividend_lat_ff <= {INT_DVEND{1'b0}} ;
			divisor_lat_ff	<= {DVSOR{1'b0}} ;
		end
		else if ( srst ) begin
			valid_ff		<= 1'b0 ;
			dividend_lat_ff <= {INT_DVEND{1'b0}} ;
			divisor_lat_ff	<= {DVSOR{1'b0}} ;
		end
		else if ( enable ) begin
			valid_ff		<= valid_i ;
			dividend_lat_ff <= dividend_i ;
			divisor_lat_ff	<= divisor_i ;
		end
	end
	
	assign valid_o			= valid_ff ;
	assign dividend_o		= dividend_out_comb ;
	assign divisor_o		= divisor_lat_ff ;
	
endmodule

`default_nettype wire
